import { Scope, DynamicScope, Environment, Handle } from '../environment';
import { DestroyableBounds } from '../bounds';
import { Tracker, UpdatableTracker } from '../builder';
import { Option, LinkedList } from '@glimmer/util';
import { IterationArtifacts } from '@glimmer/reference';
import { OpcodeJSON, UpdatingOpcode, UpdatingOpSeq } from '../opcodes';
import { Constants } from '../environment/constants';
import { DOMChanges } from '../dom/helper';
import * as Simple from '../dom/interfaces';
import VM, { CapturedStack } from './append';
export default class UpdatingVM {
    env: Environment;
    dom: DOMChanges;
    alwaysRevalidate: boolean;
    constants: Constants;
    private frameStack;
    constructor(env: Environment, {alwaysRevalidate}: {
        alwaysRevalidate?: boolean;
    });
    execute(opcodes: UpdatingOpSeq, handler: ExceptionHandler): void;
    private readonly frame;
    goto(op: UpdatingOpcode): void;
    try(ops: UpdatingOpSeq, handler: Option<ExceptionHandler>): void;
    throw(): void;
    evaluateOpcode(opcode: UpdatingOpcode): void;
}
export interface ExceptionHandler {
    handleException(): void;
}
export interface VMState {
    env: Environment;
    scope: Scope;
    dynamicScope: DynamicScope;
    stack: CapturedStack;
}
export declare abstract class BlockOpcode extends UpdatingOpcode implements DestroyableBounds {
    start: Handle;
    type: string;
    next: any;
    prev: any;
    children: LinkedList<UpdatingOpcode>;
    protected env: Environment;
    protected scope: Scope;
    protected dynamicScope: DynamicScope;
    protected stack: CapturedStack;
    protected bounds: DestroyableBounds;
    constructor(start: Handle, state: VMState, bounds: DestroyableBounds, children: LinkedList<UpdatingOpcode>);
    abstract didInitializeChildren(): void;
    parentElement(): Simple.Element;
    firstNode(): any;
    lastNode(): any;
    evaluate(vm: UpdatingVM): void;
    destroy(): void;
    didDestroy(): void;
    toJSON(): OpcodeJSON;
}
export declare class TryOpcode extends BlockOpcode implements ExceptionHandler {
    type: string;
    private _tag;
    protected bounds: UpdatableTracker;
    constructor(start: Handle, state: VMState, bounds: UpdatableTracker, children: LinkedList<UpdatingOpcode>);
    didInitializeChildren(): void;
    evaluate(vm: UpdatingVM): void;
    handleException(): void;
    toJSON(): OpcodeJSON;
}
export declare class ListBlockOpcode extends BlockOpcode {
    type: string;
    map: any;
    artifacts: IterationArtifacts;
    private lastIterated;
    private _tag;
    constructor(start: Handle, state: VMState, bounds: Tracker, children: LinkedList<UpdatingOpcode>, artifacts: IterationArtifacts);
    didInitializeChildren(listDidChange?: boolean): void;
    evaluate(vm: UpdatingVM): void;
    vmForInsertion(nextSibling: Option<Simple.Node>): VM;
    toJSON(): OpcodeJSON;
}
